/**
 * USB Device Framework
 * 
 * This file contains structures and functions
 * that are used to communicate with the usb device
 */

#ifndef FRAMEWORK_H_
#define FRAMEWORK_H_

/* USB Descriptors */

typedef struct {
    char    bLength;
    char    bDescriptorType;
    int     bcdUSB;
    char    bDeviceClass;
    char    bDeviceSubClass;
    char    bDeviceProtocol;
    char    bMaxPacketSize0;
    int     idVendor;
    int     idProduct;
    int     bcdDevice;
    char    iManufacturer;
    char    iProduct;
    char    iSerialNumber;
    char    bNumConfigurations;
} USBDeviceDescriptor;

typedef struct {
    char    bLength;
    char    bDescriptorType;
    char    wTotalLength[2];
    char    bNumInterfaces;
    char    bConfigurationValue;
    char    iConfiguration;
    char    bmAttributes;
    char    MaxPower;
} USBConfigurationDescriptor;

typedef struct {
    char    bLength;
    char    bDescriptorType;
    char    bInterfaceNumber;
    char    bAlternateSetting;
    char    bNumEndpoints;
    char    bInterfaceClass;
    char    bInterfaceSubClass;
    char    bInterfaceProtocol;
    char    iInterface;
} USBInterfaceDescriptor;

typedef struct {
    char    bLength;
    char    bDescrtiptorType;
    char    bEndpointAddress;
    char    bmAttributes;
    char    wMaxPacketSize[2];
    char    bInterval;
} USBEndpointDescriptor;

typedef struct {
    char    bLength;
    char    bDescriptorType;
    char    *wLANGID;
} USBLANGIDCodes;

typedef struct {
    char    bLength;
    char    bDescriptorType;
    char    *bString;
} USBString;

/**
 * Structure to hold a usb device
 */
typedef struct {
    USBDeviceDescriptor usbDeviceDescriptor;
    USBConfigurationDescriptor usbConfigurationDescriptor;
    int port;
    int address;
} USBDevice;

/** 
 * USBSetupATLParameters
 * Setup the ATL parameters on the ISP1362
 */
void USBSetupATLParameters(int atl_skip, int atl_last, int atl_blk_size, int atl_cnt, int atl_timeout);

/** 
 * InitializeUSB()
 * Initialize the USB hardware
 */
void InitializeUSB();

/**
 * InitializeUSBDevice()
 * Initialize the USB Device with port @port
 */
int InitializeUSBDevice(USBDevice *usbDevice, int port);

/**
 * USBDeviceGetDeviceDescriptor()
 * Gets @device's usb device descriptor
 * and returns it
 */
int USBDeviceGetDeviceDescriptor(USBDevice *device);

/**
 * USBSetupINTParameters()
 * Setup parameters for interrupt transfers
 * Returns 0 on success
 */
int USBSetupINTParameters(int int_skip, int int_last, int int_blk_size);

/**
 * USBDeviceInterruptWrite()
 * Write to @usbDevice with interrupt writes
 * @param usbDevice Structure that holds the usb device
 * @param endpoint The endpoint address to write to
 * @param payload Pointer to a integer array of the payload
 * @param psize Number of elements in @payload
 * @param pbytes Size of @payload in bytes 
 * @param freq The frequency of the interrupt
 * 
 * @return 0 on success.
 */
int USBDeviceInterruptWrite(USBDevice *usbDevice, int endpoint, int *payload, int psize, int pbytes, int freq);

/**
 * USBDeviceSetConfig()
 * Set the config of @usbDevice to value of @config
 * @param usbDevice The usb device to use
 * @param config The configuration value to set
 * @return 0 on success
 */
int USBDeviceSetConfig(USBDevice *usbDevice, int config);

#endif /*FRAMEWORK_H_*/
